#include "sintatico.h"
#include "pos_fixo.h"
#include "pos_fixo.h"
#include "tabela.h"

// Variaveis globais
int rotulo;
int endereco;
FILE *arquivo;
node *table;

Pilha p;
Pilha saida;

/*
	Funcao sintatico
	Entrada: caminho para arquivo asm a ser gerado
	Retorno: token de resultado
*/
lexico Sintatico(char* fileName)
{
	int i=0, len=0;
	char obj_fileName[256];
	lexico token;
	memset(obj_fileName, 0, 256);
	strcpy(obj_fileName,fileName);
	endereco = 1;
	rotulo = 1;
	init (p);
	init (saida);
	len = strlen(obj_fileName);
	trocarNomeArquivo(fileName);
	while( obj_fileName[len-i] != '.') {
		i++;
	}
	obj_fileName[len-i] = '\0';
	arquivo = fopen(strcat(obj_fileName, ".obj"),"w+");
	token= AnalisadorSintatico();
	fclose(arquivo);
	fecharArquivo();
	return token;
}


/*
	Funcao analisador sintatico
	Entrada: nada
	Retorno: token com resultado da compilacao
*/
lexico AnalisadorSintatico()
{
	char error[256];

	lexico token;
	node dados;
	memset(&dados, 0, sizeof(node));

	table = newTableList();
	strcpy(token.error, "Compilado com sucesso!");
	getToken(&token);

	if(token.id == SPROGRAMA) {
		GeraCodigo(-1,"START",-1,-1);
		GeraCodigo(-1,"ALLOC",0,1);
		getToken(&token);

		if( token.id == SIDENTIFICADOR) {
			token.escopo = 0;
			token.rot = rotulo;
			token.end = NULL;
			strcpy(token.tipo, PROG);
			dados.data = token;
			InsertTableItem(&table, dados); //analisador semntico
			memset(token.tipo, 0, sizeof(token.tipo));
			token.rot = NULL;
	
			getToken(&token);

			if( token.id == SPONTO_VIRGULA) {
				if(AnalisaBloco(&token) == ERROR) {
					return token;
				}				
				if( token.id == SPONTO) {
					getToken(&token);

					if (strcmp(token.lexema,"") == 0) {
						Desempilha();
						GeraCodigo(-1,"DALLOC",0,1);
						GeraCodigo(-1,"HLT",-1,-1);
						return token;
					}
					else {
						sprintf(error, "Linha %d ERRO! Codigo invalido apos fim do arquivo! \" ; \"",token.line); 
						strcpy(token.error, error);
						return token;
					}
				}
			}
			else {
				sprintf(error, "Linha %d ERRO! Operador esperado \" ; \"",token.line);
				strcpy(token.error, error);
				return token;
			}
		}
		else 
		{
			sprintf(error, "Linha %d ERRO! Identificador esperado!",token.line); 
			strcpy(token.error, error);
			return token;
		}
	}
	else
	{
		sprintf(error, "Linha %d ERRO! Palavra reservada esperada: \" SPROGRAMA \"",token.line); 
		strcpy(token.error, error);
		return token;
	}

	strcpy(token.error, "Ausncia do ponto final.");
	strcpy(token.error, error);
	return token;
}


/*
	Funcao analisa bloco, analisando o bloco por variaveis, subrotinas e comandos
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaBloco(lexico *token)
{
	getToken(token);

	if (AnalisaEtVariaveis(token) == ERROR || AnalisaSubRotinas(token) == ERROR || AnalisaComandos(token) == ERROR)
	{
		return ERROR;
	}
	else 
	{
		return SUCCESS;
	}	
}


/*
	Funcao analisa declaracao de variaveis
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaEtVariaveis(lexico *token)
{
	char error[256];

	int num_var=0, ret;

	if (token->id == SVAR)
	{
		while (token->id == SVAR)
		{
			getToken(token);

			if (token->id == SIDENTIFICADOR)
			{
				while (token->id == SIDENTIFICADOR)
				{
					ret = AnalisaVariaveis(token);

					if ( ret == ERROR)
					{
						return ERROR;
					}
					else
					{
						num_var = num_var + ret;
					}
					if (token->id == SPONTO_VIRGULA)
					{
						getToken(token);
					}
					else
					{
						sprintf(error, "Linha %d ERRO! Operador esperado \" ; \"",token->line); 
						strcpy(token->error, error);
						return ERROR;
					}
					if( num_var != 0)
		            {
	                	GeraCodigo(-1,"ALLOC",endereco,num_var);
					}
					endereco = endereco + num_var;
				}
			}
			else
			{
				sprintf(error, "Linha %d ERRO! Identificador esperado!",token->line); 
				strcpy(token->error, error);
				return ERROR;
			}
		}
	}
	return SUCCESS;
}


/*
	Funcao analisa variaveis duplicadas e pontuacao das mesmas
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaVariaveis(lexico *token)
{
	int conta_variaveis = 0;
	int end_aux = endereco;
	char error[256];

	node dados;
	memset(&dados, 0, sizeof(node));

	do{
		if(token->id == SIDENTIFICADOR)
		{
			if(PesquisaDuplicVarTabela(token) != ERROR) //PesquisaDuplicVarTabela analisador semntico
			{
				conta_variaveis++;

				token->escopo = 0;
				token->end = end_aux;
				strcpy(token->tipo,VAR);
				dados.data = *token; //analisador semntico
				end_aux++;

				InsertTableItem(&table, dados); //analisador semntico

				memset(token->tipo, 0, sizeof(token->tipo));
				token->end = NULL;

				getToken(token);

				if( token->id == SVIRGULA || token->id == SDOISPONTOS)
				{
					if( token->id == SVIRGULA )
					{
						getToken(token);

						if (token->id == SDOISPONTOS)
						{
							sprintf(error, "LINHA(%d) ERRO! Erro de Pontuao", token->line );
							strcpy(token->error, error);
							return ERROR;
						}
					}
				}
				else 
				{
					sprintf(error, "LINHA(%d) ERRO! Falta de Pontuao", token->line );
					strcpy(token->error, error);
					return ERROR;
				}
			}
			else
                        {
                                sprintf(error, "LINHA(%d) ERRO! Varivel j declarada!", token->line );
                                strcpy(token->error, error);
                                return ERROR;
                        }
		}
		else 
		{
			sprintf(error, "Linha %d ERRO! Identificador esperado!", token->line );
			strcpy(token->error, error);
			return ERROR;
		}

	}while ( token->id != SDOISPONTOS );
		
	getToken(token);

	if (AnalisaTipo(token) == ERROR)
	{

                sprintf(error, "Linha %d ERRO! Tipo declarado no existente!", token->line );
                strcpy(token->error, error);
                return ERROR;
	}

        if( conta_variaveis == 0)
        {
            sprintf(error, "Linha %d ERRO! No existem variveis declaradas!", token->line );
            strcpy(token->error, error);
            return ERROR;
        }

	return conta_variaveis;
}


/*
	Funcao analisa tipo declarado se eh inteiro ou booleano
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaTipo(lexico *token)
{
	char error[256];

	if( token->id == SINTEIRO || token->id == SBOOLEANO) {
		ColocaTipoTabela(token); //Analisador Semntico 
	}
	else {
		sprintf(error, "Linha %d ERRO! Tipo da variavel invalido!!", token->line );
		strcpy(token->error, error);
		return ERROR;
	}
		
	getToken(token);
	return SUCCESS;
}


/*
	Funcao analisa comandos, a sua presenca e pontuacao
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaComandos(lexico *token)
{
	char error[256];
	char identificador[256];

	if(token->id == SINICIO)
	{
		getToken(token);

		if(AnalisaComandoSimples(token) == ERROR)
		{
			return ERROR;
		}

		while(token->id != SFIM)
		{
			if(token->id == SPONTO_VIRGULA)
			{
				getToken(token);

				if( token->id != SFIM )
				{
					if(AnalisaComandoSimples(token) == ERROR)
					{
						return ERROR;
					}
				}
			}
			else
			{
				sprintf(error, "Linha %d ERRO! Ausncia do operador ponto e virgula\" ; \"",token->line); 
				strcpy(token->error, error);
				return ERROR;
			}
			
		}
		getToken(token);
	}
	else
	{
		sprintf(identificador, " \" %s \"", token->lexema);
		sprintf(error, "LINHA(%d) ERRO! Operador invlido: ", token->line);
		strcat(error, identificador);
		strcpy(token->error, error); 
		return ERROR;
	}
	return SUCCESS;
}

/*
	Funcao analisa comando simples
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaComandoSimples(lexico *token)
{
	if(token->id == SIDENTIFICADOR)
	{
		return AnalisaAtribChProcedimento(token); 
	}
	else if (token->id == SSE)
	{
		return AnalisaSe(token); 
	}
	else if (token->id == SENQUANTO)
	{
		return AnalisaEnquanto(token);
	}
	else if (token->id == SLEIA)
	{
		return AnalisaLeia(token);
	}
	else if (token->id == SESCREVA)
	{
		return AnalisaEscreva(token);
	}
	else
	{
		return AnalisaComandos(token);
	}
}


/*
	Funcao analisa atribuicao e chamada de procedimento, incluindo compatibilidade de tipos, geracao de pos fixo
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaAtribChProcedimento(lexico *token)
{
        char error[256];

	int endereco_memoria, ret;

	lexico ant = *token;

	getToken(token);

	if (token->id == SATRIBUICAO)
	{
		getToken(token);

                if (PesquisaDeclVarTabela(ant.lexema) != SUCCESS && PesquisaDeclVarFuncTabela(ant.lexema) != SUCCESS)
                {
                        sprintf(error, "Linha %d ERRO! Fator no declarado '%s'!",token->line, ant.lexema);
                        strcpy(token->error, error);
                        return ERROR;
                }

		if (AnalisaExpressao(token) == ERROR)
		{
			return ERROR;
		}

		ret = AnalisaExpressaoPosFixa();

		ConsultaTipo(&ant);

		//verifica se a atribuio  feita entre variaveis e/ou valores de mesmo tipo 
		if( ret == SBOOLEANO )
		{
			if( strstr(ant.tipo, "BOOL") == NULL )
			{
				sprintf(error, "Linha %d ERRO! Tipo invlido para a operao. Esperado BOOLEANO!",token->line);
		                strcpy(token->error, error);
		                return ERROR;
			}
		}
		else if ( ret == SINTEIRO )
		{
			if( strstr(ant.tipo, "INT") == NULL)
			{
				sprintf(error, "Linha %d ERRO! Tipo invlido para a operao. Esperado INTEIRO!",token->line);
		                strcpy(token->error, error);
		                return ERROR;
			}

		}
		else 
		{
			return ERROR;
		}

		GeraCodigoPosFixo();

		endereco_memoria = ConsultaEndereco(ant.lexema);

		GeraCodigo(-1,"STR",endereco_memoria,-1);
	}
	else
	{
		if (AnalisaChamadaProcedimento(&ant) == ERROR)
		{
                        strcpy(token->error, ant.error); 
			return ERROR;
		}
	}
	return SUCCESS;
}


/*
	Funcao analisa comando leia
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaLeia(lexico *token)
{
	int endereco_memoria;
	char error[256];

	getToken(token);

	if (token->id == SABRE_PARENTESES)
	{
		getToken(token);
		
		if (token->id == SIDENTIFICADOR)
		{
			if (PesquisaDeclVarTabela(token->lexema) == SUCCESS)
			{
                                ConsultaTipo(token);

                                if( strcmp(token->tipo, VINT) == 0)
                                {
				        endereco_memoria = ConsultaEndereco(token->lexema);

				        GeraCodigo(-1,"RD",-1,-1);

				        GeraCodigo(-1,"STR",endereco_memoria,-1);

			        	getToken(token);

			        	if (token->id == SFECHA_PARENTESES)
				        {
					        getToken(token);
				        }
				        else
				        {
					        sprintf(error, "Linha %d ERRO! Fecha parenteses esperado \" ) \"!",token->line);
					        strcpy(token->error, error);
					        return ERROR;
				        }
                                }
                                else
                                {
                                        sprintf(error, "Linha %d ERRO! A funo escreva aceita apenas tipo INTEIRO de parmetro!",token->line);
                                        strcpy(token->error, error);
                                        return ERROR;
                                }
			}
			else
				return ERROR;
		}
		else
		{
			sprintf(error, "Linha %d ERRO! Identificador esperado!",token->line); 
			strcpy(token->error, error);
			return ERROR;
		}
	}
	else
	{
		sprintf(error, "Linha %d ERRO! Abre parenteses esperado \" ( \"!",token->line); 
		strcpy(token->error, error);
		return ERROR;
	}

	return SUCCESS;
}


/*
	Funcao analisa comando escreva
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaEscreva(lexico *token)
{
	int endereco_memoria;
	int end_rotulo;
	char error[256];

	getToken(token);

	if (token->id == SABRE_PARENTESES)
	{
		getToken(token);

		if (token->id == SIDENTIFICADOR)
		{
			if(PesquisaDeclVarFuncTabela(token->lexema) == SUCCESS)
			{
                                ConsultaTipo(token);

                                if(strcmp(token->tipo, VINT) == 0 || strcmp(token->tipo, FINT) == 0)
                                {
				        endereco_memoria = ConsultaEndereco(token->lexema);

				        if (ConsultaTipo(token) == FUNCAO)
				        {
				        	end_rotulo = ConsultaRotulo(token->lexema);

				                GeraCodigo(-1,"CALL",end_rotulo,-1);
				        }

				        GeraCodigo(-1,"LDV",endereco_memoria,-1);

				        GeraCodigo(-1,"PRN",-1,-1);

				        getToken(token);

				        if (token->id == SFECHA_PARENTESES)
				        {
					        getToken(token);
				        }
                                        else
                                        {
                                                sprintf(error, "Linha %d ERRO! Fecha parenteses esperado \" ) \"!",token->line);
				        	strcpy(token->error, error);
				        	return ERROR;
				        }
                                }
                                else
        			{
	        			sprintf(error, "LINHA(%d) ERRO! Tipo errado (BOOLEANO)! Tipo INTEIRO esperado!",token->line);
		        		strcpy(token->error, error);
			        	return ERROR;
        			}
			}
			else
				return ERROR;
		}
		else
		{
			sprintf(error, "Linha %d ERRO! Identificador esperado!",token->line);
			strcpy(token->error, error);
			return ERROR;
		}
	}
	else
	{
		sprintf(error, "Linha %d ERRO! Abre parenteses esperado \" ( \"!",token->line);
		strcpy(token->error, error);
		return ERROR;
	}

	return SUCCESS;
}


/*
	Funcao analisa enquanto
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaEnquanto(lexico *token)
{
	char error[256];
	char operador[256];
	int auxrot1, auxrot2;

	auxrot1 = rotulo;

	GeraCodigo(rotulo,"NULL",-1,-1); //inicio do while

	rotulo = rotulo + 1;

	getToken(token);

	if(AnalisaExpressao(token) == ERROR)
	{
		return ERROR;
	}

	if(AnalisaExpressaoPosFixa() == SBOOLEANO)
	{
		GeraCodigoPosFixo();
	}
	else
	{
                sprintf(error, "Linha %d ERRO! Tipo invlido para a operao. Esperado BOOLEANO!",token->line);
		strcpy(token->error, error);
		return ERROR;
	}
	if(token->id == SFACA)
	{
		auxrot2 = rotulo;
		
		GeraCodigo(-1,"JMPF",rotulo,-1); //salta se falso

		rotulo = rotulo + 1;

		getToken(token);

		if(AnalisaComandoSimples(token) == ERROR)
		{
			return ERROR;
		}

		GeraCodigo(-1,"JMP",auxrot1,-1); //retorna para o inicio do loop

		GeraCodigo(auxrot2,"NULL",-1,-1); //fim do while
	}
	else
	{
		sprintf(operador, "\" %s \"", token->lexema);
		sprintf(error, "LINHA(%d) ERRO! Faca esperado! Encontrado: ", token->line);
		strcat(error, operador);
		strcpy(token->error, error);
		return ERROR;
	}
	return SUCCESS;
}


/*
	Funcao pesquisa declaracao de variaveis e funcoes
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int PesquisaDeclVarFuncTabela(char *lexema)
{
	char error[256];

	node *p;

	p = table;

	while(p->next != NULL)
	{
		if(strcmp(p->data.lexema, lexema) == 0)
		{
			if(strcmp(p->data.tipo, VINT) == 0 || strcmp(p->data.tipo, FINT) == 0)
			{
				return SUCCESS;
			}
			else if(strcmp(p->data.tipo, VBOOL) == 0 || strcmp(p->data.tipo, FBOOL) == 0)
			{
				return SUCCESS;
			}
			else
			{
				sprintf(error, "LINHA(%d) ERRO! Nao e uma variavel nem uma funcao!", p->data.line);
				strcpy(p->data.error, error);
				return ERROR;
			}
		}
		p = p->next;
	}
	sprintf(error, "LINHA(%d) ERRO! Nao foi encontrado nem uma variavel nem uma funcao com este nome ( %s )!", p->data.line, lexema);
        strcpy(p->data.error, error);
	return ERROR;
}


/*
	Pesquisa declaracao variaveis
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int PesquisaDeclVarTabela(char *lexema)
{
	char error[256];

	node *p;

	p = table;

	while(p->next != NULL)
	{
		if(strcmp(p->data.lexema, lexema) == 0)
		{
			if (strcmp(p->data.tipo, VINT) == 0)
			{
				return SUCCESS;
			}
			else if (strcmp(p->data.tipo, VBOOL) == 0)
			{
				return SUCCESS;
			}
			else
			{
				sprintf(error, "LINHA(%d) ERRO! ( %s ) Nao e uma variavel! ", p->data.line, p->data.lexema);
				strcpy(p->data.error, error);
				return ERROR;
			}
		}
		p = p->next;
	}
	sprintf(error, "ERRO! ( %s ) Nao foi declarada! ", p->data.lexema);
	strcpy(p->data.error, error);
	return ERROR;
}

/*
	Pesquisa declaracao duplicacao de variaveis
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int PesquisaDuplicVarTabela(lexico *token)
{
	char error[256];

	lexico *dado;
	node *p;
	int flag = 0;

	p = table;

	while(p->next != NULL)
	{
		dado = &p->data;

		if(dado->escopo == 1)
		{
			flag = 1;
		}
		if(strcmp(dado->lexema, token->lexema) == 0)
		{
			if (strcmp(p->data.tipo, VBOOL) == 0 || strcmp(p->data.tipo, VINT) == 0)
			{
				if(flag != 1)
				{
					sprintf(error, "LINHA(%d) ERRO! ( %s ) Variavel ja declarada! ", p->data.line, p->data.lexema);
					strcpy(p->data.error, error);
					return ERROR;
				}
				flag = 1;
			}
			else
			{
				sprintf(error, "LINHA(%d) ERRO! ( %s ) ja foi utilizado! A variavel nao pode ser declarada com esse nome!", p->data.line, p->data.lexema);
				strcpy(p->data.error, error);
				return ERROR;
			}
		}
		p = p->next;
	}
	return SUCCESS;
}


/*
	Funcao analisa chamado de procedimento, consultando seu rotulo
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaChamadaProcedimento(lexico *token)
{
	int end_rotulo;
	char error[256];
	char operador[256];

	if(token->id != SIDENTIFICADOR)
	{		
		sprintf(operador, "( %s )", token->lexema);
		sprintf(error, "LINHA(%d) ERRO! Operador no identificado ", token->line);
		strcat(error, operador);
		strcpy(token->error, error);
		return ERROR;
	}
	else
	{
                if(PesquisaDeclProcTabela(token) != TRUE)
                {
                        sprintf(operador, "( %s )", token->lexema);
		        sprintf(error, "LINHA(%d) ERRO! Identificador no declarado! ", token->line);
		        strcat(error, operador);
		        strcpy(token->error, error);
		        return ERROR;
                }
		end_rotulo = ConsultaRotulo(token->lexema);

		GeraCodigo(-1,"CALL",end_rotulo,-1);
		//GeraCodigo(-1,"LDV",0,-1);
	}
	return SUCCESS;
}


/*
	Funcao analisa chamada de funcao, consulta se o nome da funcao eh valido
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaChamadaFuncao(lexico *token)
{
	int end_rotulo;
	int endereco_memoria;
	char error[256];
	char operador[256];

	if(token->id != SIDENTIFICADOR)
	{
		sprintf(operador, "( %s )", token->lexema);
		sprintf(error, "LINHA(%d) ERRO! Operador no identificado ", token->line);
		strcat(error, operador);
		strcpy(token->error, error);
		return ERROR;
	}
	else
	{
                if(PesquisaDeclFuncTabela(token) != TRUE)
                {
                        sprintf(operador, "( %s )", token->lexema);
		        sprintf(error, "LINHA(%d) ERRO! Identificador no declarado! ", token->line);
		        strcat(error, operador);
		        strcpy(token->error, error);
		        return ERROR;
                }

		end_rotulo = ConsultaRotulo(token->lexema);
		endereco_memoria = ConsultaEndereco(token->lexema);

                getToken(token);
//                GeraCodigo(-1,"CALL",end_rotulo,-1);
//                GeraCodigo(-1,"LDV",endereco_memoria,-1);
	}

	return SUCCESS;
}


/*
	Funcao analisa comando se
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaSe(lexico *token)
{
	int auxrot1, auxrot2;
	char error[256];
	
	getToken(token);

	if(AnalisaExpressao(token) == ERROR)
	{
		return ERROR;
	}
	if(AnalisaExpressaoPosFixa() == SBOOLEANO)
	{
		GeraCodigoPosFixo();
	}
	else
	{
		sprintf(error, "Linha %d ERRO! Tipo invlido para a operao. Esperado BOOLEANO!",token->line);
		strcpy(token->error, error);
		return ERROR;
	}

	auxrot1 = rotulo;

	GeraCodigo(-1,"JMPF",rotulo,-1);

	rotulo = rotulo + 1;

	if( token->id == SENTAO )
	{
		getToken(token);

		if(AnalisaComandoSimples(token) == ERROR)
		{
			return ERROR;
		}

		if(token->id == SSENAO)
		{
			auxrot2 = rotulo;

			GeraCodigo(-1,"JMP",rotulo,-1);

			rotulo = rotulo + 1;

			GeraCodigo(auxrot1,"NULL",-1,-1);

			getToken(token);

			if(AnalisaComandoSimples(token) == ERROR)
			{
				return ERROR;
			}

			GeraCodigo(auxrot2,"NULL",-1,-1);
		}
		else
		{
			GeraCodigo(auxrot1,"NULL",-1,-1);
		}
	
	}
	else
	{
		sprintf(error, "LINHA(%d) ERRO! Esperado operador ENTAO, encontrado ( %s )!", token->line, token->lexema);
		strcpy( token->error, error);
		return ERROR;
	}
	return SUCCESS;
}


/*
	Funcao analisa sub rotinas, analisa declaracao de chamada de procedimento, funcao
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaSubRotinas(lexico *token)
{
	int auxrot = -1;
	char error[256];

	while (token->id == SPROCEDIMENTO || token->id == SFUNCAO)
	{

		if (token->id == SPROCEDIMENTO)
		{
                        auxrot = rotulo;

                        GeraCodigo(-1,"JMP",rotulo,-1); //salta sub-rotinas

                        rotulo = rotulo + 1;
                        GeraCodigo(rotulo,"NULL",-1,-1); //salta sub-rotinas
                        rotulo = rotulo + 1;
                        
			if (AnalisaDeclaracaoDeProcedimento(token) == ERROR)
			{
				return ERROR;
			}

		}
		else
		{
                        auxrot = AnalisaDeclaracaoDeFuncao(token);
			if (auxrot == ERROR)
			{
				return ERROR;
			}
		}
	
		if (token->id == SPONTO_VIRGULA)
		{
			getToken(token);
		}
		else
		{
			sprintf(error, "Linha %d ERRO! Ausencia do operador ponto e virgula \" ; \"!",token->line); 
			strcpy(token->error, error);
			return ERROR;
		}

                if( auxrot != -1 && auxrot != rotulo )
               	{
	       	        GeraCodigo(auxrot,"NULL",-1,-1); //inicio do principal
	        }
	}
	return SUCCESS;
}


/*
	Funcao coloca tipo tabela, seta os tipos validos na tabela de simbolos
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int ColocaTipoTabela(lexico *token)
{
	char error[256];

	node *p;

	p = table;

	while (p->next != NULL)
	{
		if (strcmp(p->data.tipo, VAR) == 0)
		{
			if (strcmp(token->lexema, "inteiro") == 0)
			{
				strcpy(p->data.tipo,VINT);
			}
			else if (strcmp(token->lexema, "booleano") == 0)
			{
				strcpy(p->data.tipo,VBOOL);
			}
			else
			{
				sprintf(error, "Linha %d ERRO! Tipo invalido! ( %s )",token->line, token->lexema); 
				strcpy(token->error, error);
				return ERROR;
			}
		}
		p = p->next;
	}
	return SUCCESS;
}


/*
	Funcao analisa expressao e compatibilidade de simbolos na expressao
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaExpressao(lexico *token)
{
	if (AnalisaExpressaoSimples(token) == ERROR)
	{
		return ERROR;
	}

	if (token->id == SMAIORIG || token->id == SMAIOR || token->id == SMENOR 
		  || token->id == SIG || token->id == SMENORIG || token->id == SDIF)
	{
		strcpy(token->tipo, CMP); //identifica o tokem como sendo um operando de comparao 

		empilhar(token);

		getToken(token);

		if(AnalisaExpressaoSimples(token) == ERROR)
		{
			return ERROR;
		}
	}
	return SUCCESS;
}


/*
	Funcao analisa expressao simples, verifica operadores unarios ou binarios na expressao
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaExpressaoSimples(lexico *token)
{
	lexico token_aux;
	memset(&token_aux,0,sizeof (lexico));

	if (token->id == SMAIS || token->id == SMENOS)	//  O OPERADOR UNARIO
	{

		strcpy(token->tipo, UNARIO); //idenntifica o token como sendo um operador aritmtico
		empilhar(token);
		token_aux = *token;
		strcpy(token_aux.tipo,UNARIO);
		getToken(token);
	}
		
	if (AnalisaTermo(token) == ERROR)
		return ERROR;
	
	while (token->id == SMAIS || token->id == SMENOS || token->id == SOU)
	{

		strcpy(token->tipo, ART); //idenntifica o token como sendo um operador aritmtico
		empilhar(token);
		getToken(token);

		if(AnalisaTermo(token)==ERROR)
		{
			return ERROR;
		}
	}
	return SUCCESS;
}


/*
	Funcao analisa termo
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaTermo(lexico *token)
{
	if (AnalisaFator(token) == ERROR)
	{
		return ERROR;
	}

	while (token->id == SMULT || token->id == SDIV || token->id == SE )
	{
		strcpy(token->tipo, ART); //idenntifica o token como sendo um operador aritmtico
		empilhar(token);
		getToken(token);

		if (AnalisaFator(token) == ERROR)
		{
			return ERROR;
		}
	}
	return SUCCESS;
}


/*
	Funcao analisa o conteudo do fator e a sintaxe
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaFator(lexico *token)
{
	if (token->id == SIDENTIFICADOR)
	{
		empilhar(token);

		if (PesquisaDeclFuncTabela(token) == TRUE)
		{
			if (AnalisaChamadaFuncao(token) == ERROR)
			{
				return ERROR;
			}
		}
		else if( PesquisaDeclVarTabela(token->lexema) == SUCCESS )
                {
			getToken(token);
                }
                else if (strcmp(token->lexema,"VERDADEIRO") == 0 || strcmp(token->lexema,"FALSO") == 0 ||
                         strcmp(token->lexema,"verdadeiro") == 0 || strcmp(token->lexema,"falso") == 0)
	        {
		        getToken(token);
	        }
                else
                {
                        char error[256], operador[256];
			sprintf(operador, "\" %s \" !", token->lexema);
			sprintf(error, "LINHA(%d) ERRO! Varivel no declarada ", token->line);
			strcat(error, operador);
			strcpy(token->error, error);
			return ERROR;
                }
	}
	else if (token->id == SNUMERO)
	{
		empilhar(token);

		getToken(token);
	}
	else if (token->id == SNAO)
	{

		strcpy(token->tipo, UNARIO); //idenntifica o token como sendo um operador aritmtico
		empilhar(token);
		getToken(token);

		if (AnalisaFator(token) == ERROR)
		{
			return ERROR;
		}
	}
	else if (token->id == SABRE_PARENTESES)
	{
		empilhar(token);
		getToken(token);

		if (AnalisaExpressao(token)==ERROR)
		{
			return ERROR;
		}
		if (token->id == SFECHA_PARENTESES)
		{
			empilhar(token);

			getToken(token);
		}
		else
		{
			char error[256], operador[256];
			sprintf(operador, "\" %s \"", token->lexema);
			sprintf(error, "LINHA(%d) ERRO! Operador no identificado!", token->line);
			strcat(error, operador);
			strcpy(token->error, error);
			return ERROR;
		}
	}
	else if (strcmp(token->lexema,"VERDADEIRO") == 0 || strcmp(token->lexema,"FALSO") == 0 ||
                 strcmp(token->lexema,"verdadeiro") == 0 || strcmp(token->lexema,"falso") == 0)
	{

                strcpy(token->tipo, VBOOL);
                empilhar(token);
                getToken(token);
	}
	else
	{
		char error[256], fator[256];
		sprintf(fator, "\" %s \"", token->lexema);
		sprintf(error, "LINHA(%d) ERRO! Fator no identificado!", token->line);
		strcat(error, fator);
		strcpy(token->error, error);
		return ERROR;
	}
	//gerar cdigo AQUI, no retorno
	return SUCCESS;
}


/*
	Funcao analisa declaracao de procedimento, se ja nao tiver declarado, insere ele na tabela
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaDeclaracaoDeProcedimento(lexico *token)
{

	node dados;
	memset(&dados, 0, sizeof(node));

	getToken(token);

	token->escopo = 1;

	if (token->id == SIDENTIFICADOR)
	{
		if (PesquisaDeclProcTabela(token) == FALSE)
		{
			token->rot = rotulo - 1;
			strcpy(token->tipo, PROC);
			dados.data = *token;

			InsertTableItem(&table, dados);
		       //	GeraCodigo(rotulo,"NULL",-1,-1);
		       //	rotulo = rotulo + 1;

			memset(token->tipo, 0, sizeof(token->tipo));
			token->rot = NULL;		

			getToken(token);

			if (token->id == SPONTO_VIRGULA)
			{
				if(AnalisaBloco(token) == ERROR)
				{
					return ERROR;
				}
			}
			else
			{
				char error[256];
				sprintf(error, "Linha %d ERRO! Ausencia do operador \" ; \"!",token->line);
				strcpy(token->error, error);
				return ERROR;
			}
		}
		else
                {
                        char error[256];
		       	sprintf(error, "Linha %d ERRO! Procedimento ja declarado!",token->line);
		       	strcpy(token->error, error);
			return ERROR;
	}
	}
	else
        {
                char error[256];
		sprintf(error, "Linha %d ERRO! Esperado um nome de procedimento! Encontrado ( %s )!",token->line, token->lexema);
		strcpy(token->error, error);
		return ERROR;
        }

	Desempilha();

	GeraCodigo(-1,"RETURN",-1,-1);

	return SUCCESS;
}


/*
	Funcao analisa declaracao de funcao, se nao tiver declarado, inseri na tabela
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaDeclaracaoDeFuncao(lexico *token)
{
	node dados;
        int aux_rot = 0;
	memset(&dados, 0, sizeof(node));

	getToken(token);

	token->escopo = 1;
	
	if (token->id == SIDENTIFICADOR)
	{
		if (PesquisaDeclFuncTabela(token) == FALSE)
		{

			GeraCodigo(-1,"JMP",rotulo,-1); //salta sub-rotinas
			aux_rot = rotulo;
			rotulo = rotulo + 1;
			
			GeraCodigo(rotulo,"NULL",-1,-1); //salta sub-rotinas

			rotulo = rotulo + 1;

			token->rot = rotulo - 1;
			token->end = 0;
			strcpy(token->tipo, FUNC);
			dados.data = *token;

			InsertTableItem(&table, dados);

			memset(token->tipo, 0, sizeof(token->tipo));
			token->rot = NULL;
			token->end = NULL;

			getToken(token);

			if (token->id == SDOISPONTOS)
			{
				getToken(token);

				if (token->id == SINTEIRO || token->id == SBOOLEANO)
				{
					if (token->id == SINTEIRO)
					{
						strcpy(table->data.tipo, FINT);
					}
					else
					{
						strcpy(table->data.tipo, FBOOL);
					}

					getToken(token);

					if (token->id == SPONTO_VIRGULA)
					{
						if(AnalisaBloco(token) == ERROR)
						{
							return ERROR;
						}
					}
				}
				else
                                {
					char error[256];
				        sprintf(error, "Linha %d ERRO! Esperado um tipo para a funcao apos o \" : \"!",token->line);
				        strcpy(token->error, error);
					return ERROR;
			}
			}
			else
                        {
                                char error[256];
				sprintf(error, "Linha %d ERRO! Ausencia do operador \" : \"!",token->line);
				strcpy(token->error, error);
				return ERROR;
		}
		}
		else
			return ERROR;
	}
	else
	{
                char error[256];
		sprintf(error, "Linha %d ERRO! Esperado um nome de funcao! Encontrado ( %s )!",token->line, token->lexema);
		strcpy(token->error, error);
		return ERROR;
        }
	Desempilha();

	GeraCodigo(-1,"RETURN",-1,-1);

	return aux_rot;
}


/*
	Funcao pesquisa declaracao procedimento na tabela
	Entrada: token
	Retorno: bool true ou false
*/
bool PesquisaDeclProcTabela(lexico *token)
{
	node *p;

	p = table;

	while (p->next != NULL)
	{
		if (strcmp(p->data.lexema, token->lexema) == 0)
		{
			if (strcmp(p->data.tipo, PROC) == 0)
			{
					return TRUE;
			}
			else
			{
				return FALSE;
			}
		}
		p = p->next;
	}
	return FALSE;
}


/*
	Funcao pesquisa declaracao funcao na tabela
	Entrada: token
	Retorno: true ou false
*/
bool PesquisaDeclFuncTabela(lexico *token)
{
	node *p;

	p = table;

	while (p->next != NULL)
	{
		if (strcmp(p->data.lexema, token->lexema) == 0)
		{
			if (strcmp(p->data.tipo, FBOOL) == 0 || strcmp(p->data.tipo, FINT) == 0)
			{
					return TRUE;
			}
			else
			{
				return FALSE;
			}
		}
		p = p->next;
	}
	return FALSE;
}


/*
	Funcao desempilha, usado no fim, para desalocar variaveis, funcao e procedimento
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
void Desempilha()
{
	int num_var = 0;

	while (table->data.escopo != 1)
	{
		if (strcmp(table->data.tipo, VINT) == 0 || strcmp(table->data.tipo, VBOOL) == 0 )
		{
			num_var++;
		}

		if (table->next == NULL)
		{
			break;
		}

		DeleteTableItem(&table);
	}

	table->data.escopo = 0;

	if (num_var != 0)
	{
		GeraCodigo(-1,"DALLOC",endereco-num_var,num_var);
		endereco = endereco - num_var;
	}
}


/*
	Funcao consulta rotulo, de acordo com o  nome
	Entrada: lexema
	Retorno: inteiro: rotulo
*/
int ConsultaRotulo(char *lexema)
{
	node *p;

	p = table;

	while (p->next != NULL)
	{
		if (strcmp(p->data.lexema, lexema) == 0)
		{
				return p->data.rot;
		}
		p = p->next;
	}
	return ERROR;
}


/*
	Funca consulta endereco de uma variavel
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int ConsultaEndereco(char *lexema)
{
	node *p;

	p = table;

	while (p->next != NULL)
	{
		if (strcmp(p->data.lexema, lexema) == 0)
		{
				return p->data.end;
		}
		p = p->next;
	}
	return ERROR;
}


/*
	Funcao consulta tipo do lexema na tabela
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int ConsultaTipo(lexico *token)
{
	node *p;

	p = table;


	while (p->next != NULL)
	{
		if(strcmp(p->data.lexema, token->lexema) == 0)
		{
			if (strcmp(p->data.tipo, VBOOL) == 0 || strcmp(p->data.tipo, VINT) == 0)
			{

				//coloca o tipo do identificador no token retornando o por parametro
				if(strcmp(p->data.tipo, VBOOL) == 0)
				{
					strcpy(token->tipo, VBOOL);
				}
				else
				{
					strcpy(token->tipo, VINT);
				}

				return VARIAVEL;
			}
			if (strcmp(p->data.tipo, FBOOL) == 0 || strcmp(p->data.tipo, FINT) == 0)
			{

				//coloca o tipo do identificador no token retornando o por parametro
				if(strcmp(p->data.tipo, FBOOL) == 0)
				{
					strcpy(token->tipo, FBOOL);
				}
				else
				{
					strcpy(token->tipo, FINT);
				}

				return FUNCAO;
			}
		}
		p = p->next;
	}
        return ERROR;
}


/*
	Funcao empilhar, usado na tranformacao para pos fixo,  ordenado
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
void empilhar(lexico *token)
{
	if(token->id == SABRE_PARENTESES)
	{
		push(p, *token);
	}
	else if(token->id == SFECHA_PARENTESES)
	{
		while (topo(p).id != SABRE_PARENTESES)
		{
			push(saida, pop(p));//DESEMPILHA E poe na saida enquanto diferente de (
		}
		pop(p);//desempilha o (
		if (p.top->info.lexema != NULL)
		{
			if (topo(p).tipo == UNARIO || topo(p).id == SNAO)
			{
				push(saida, pop(p));
			}
		}
	}
	else if (token->id == SNAO || token->tipo == UNARIO)
	{
		push(p, *token);//poe na pilha
	}
	else if (token->id == SE || token->id == SOU)
	{
		if (p.top->info.lexema != NULL)
		{
			while (topo(p).id == SE || topo(p).id == SOU || topo(p).id == SMAIOR || topo(p).id == SMAIORIG || 
				   topo(p).id == SIG || topo(p).id == SMENOR || topo(p).id == SMENORIG || topo(p).id == SDIF || 
				   topo(p).id == SMULT || topo(p).id == SDIV || topo(p).id == SMAIS || topo(p).id == SMENOS)
			{
				push(saida, pop(p));

				if (p.top->info.lexema == NULL)
				{
					break;
				}
			}
		}
		push(p, *token);
	}
	else if (token->id == SMULT || token->id == SDIV)
	{
		if (p.top->info.lexema != NULL)
		{
			while (topo(p).id == SMULT || topo(p).id == SDIV)
			{
				push(saida, pop(p));//tira da pilha e poe na saida

				if (p.top->info.lexema == NULL)
				{
					break;
				}
			}
		}
		push(p, *token);//poe na pilha
	}
	else if (token->id == SMAIS || token->id == SMENOS)
	{
		if (p.top->info.lexema != NULL)
		{
			while (topo(p).id == SMULT || topo(p).id == SDIV || topo(p).id == SMAIS || topo(p).id == SMENOS)
			{
				push(saida, pop(p));//tira da pilha e poe na saida

				if (p.top->info.lexema == NULL)
				{
					break;
				}
			}
		}
		push(p, *token);//poe na pilha
	}
	else if (token->id == SMAIOR || token->id == SMAIORIG || token->id == SIG || 
		     token->id == SMENOR || token->id == SMENORIG || token->id == SDIF)
	{
		if(p.top->info.lexema != NULL)
		{
			while (topo(p).id == SMAIOR || topo(p).id == SMAIORIG || topo(p).id == SIG || 
				   topo(p).id == SMENOR || topo(p).id == SMENORIG || topo(p).id == SDIF || 
				   topo(p).id == SMULT || topo(p).id == SDIV || topo(p).id == SMAIS || topo(p).id == SMENOS)
			{
				push(saida, pop(p));//tira da pilha e poe na saida

				if (p.top->info.lexema == NULL)
				{
					break;
				}
			}
		}
		push(p, *token);//poe na pilha
	}
	else //SIDENTIFICADOR
	{

		strcpy(token->tipo, VAR);

		push(saida, *token);
		
		if (p.top->info.lexema != NULL)
		{
			if (strcmp(topo(p).tipo, UNARIO) == 0 || topo(p).id == SNAO)
			{
				push(saida, pop(p));
			}
		}
	}

	memset(token->tipo, 0, sizeof(token->tipo));
}


/*
	Funcao gera codigo
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
void GeraCodigo(int rotulo_rec, char *instrucao_rec, int aux1_rec, int aux2_rec)
{
	if (rotulo_rec == -1)
	{
		fprintf(arquivo,"    ");
	}
	else
	{
		if (rotulo_rec >= 0 && rotulo_rec < 10)
		{
			fprintf(arquivo,"%d   ",rotulo_rec);
		}
		else if (rotulo_rec >= 10 && rotulo_rec < 100)
		{
			fprintf(arquivo,"%d  ",rotulo_rec);
		}
		else if (rotulo_rec >= 100 && rotulo_rec < 1000)
		{
			fprintf(arquivo,"%d ",rotulo_rec);
		}
		else
		{
			fprintf(arquivo,"%d",rotulo_rec);
		}
	}
	if (strcmp(instrucao_rec,"-1") == 0)
	{
		fprintf(arquivo,"        ");
	}
	else
	{
		if (strlen(instrucao_rec) == 1)
		{
			fprintf(arquivo,"%s       ",instrucao_rec);
		}
		else if (strlen(instrucao_rec) == 2)
		{
			fprintf(arquivo,"%s      ",instrucao_rec);
		}
		else if (strlen(instrucao_rec) == 3)
		{
			fprintf(arquivo,"%s     ",instrucao_rec);
		}
		else if (strlen(instrucao_rec) == 4)
		{
			fprintf(arquivo,"%s    ",instrucao_rec);
		}
		else if (strlen(instrucao_rec) == 5)
		{
			fprintf(arquivo,"%s   ",instrucao_rec);
		}
		else if (strlen(instrucao_rec) == 6)
		{
			fprintf(arquivo,"%s  ",instrucao_rec);
		}
		else if (strlen(instrucao_rec) == 7)
		{
			fprintf(arquivo,"%s ",instrucao_rec);
		}
		else
		{
			fprintf(arquivo,"%s",instrucao_rec);
		}
	}
	if (aux1_rec == -1)
	{
		fprintf(arquivo,"    ");
	}
	else
	{
		if (aux1_rec >= 0 && aux1_rec < 10)
		{
			fprintf(arquivo,"%d   ",aux1_rec);
		}
		else if (aux1_rec >= 10 && aux1_rec < 100)
		{
			fprintf(arquivo,"%d  ",aux1_rec);
		}
		else if (aux1_rec >= 100 && aux1_rec < 1000)
		{
			fprintf(arquivo,"%d ",aux1_rec);
		}
		else
		{
			fprintf(arquivo,"%d",aux1_rec);
		}
	}
	if (aux2_rec == -1)
	{
		fprintf(arquivo,"    ");
	}
	else
	{
		if (aux2_rec >= 0 && aux2_rec < 10)
		{
			fprintf(arquivo,"%d   ",aux2_rec);
		}
		else if (aux2_rec >= 10 && aux2_rec < 100)
		{
			fprintf(arquivo,"%d  ",aux2_rec);
		}
		else if (aux2_rec >= 100 && aux2_rec < 1000)
		{
			fprintf(arquivo,"%d ",aux2_rec);
		}
		else
		{
			fprintf(arquivo,"%d",aux2_rec);
		}
	}
	fprintf(arquivo,"\n");
}


/*
	Funcao analisa expressao pos fixa
	Entrada: token
	Retorno: inteiro: -1para falha e 1 para sucesso
*/
int AnalisaExpressaoPosFixa()
{
	
	no *inicio_expressao, *aux;
	Pilha expressao;
        char error[256];

	init(expressao);
	//if para verificar se no restou nada na pilha aps transformao da expressao
	//caso isso seja verdade retirar da pilha e colocar na pilha de saida, a qual contera
	//a expresso no formato ps fixo

	if( p.top != NULL)
	{
		while(p.top != NULL)
		{
			push(saida, p.top->info);

			pop(p);
		}
	}

	DuplicarInvPilha(saida, expressao);
	inicio_expressao = expressao.top;
	inicio_expressao->linkBack = NULL;
	aux = inicio_expressao;
	ConsultaTipo(&expressao.top->info);

        if(((strstr(expressao.top->info.tipo, "INT") != NULL) || (expressao.top->info.id == SNUMERO)) && expressao.top->link == NULL)
        {
                return SINTEIRO;
        }
	else if ( (strstr(expressao.top->info.tipo, "BOOL") != NULL || expressao.top->info.id == SVERDADEIRO ||
                   expressao.top->info.id == SVERDADEIRO)&& expressao.top->link == NULL)
        {
		return SBOOLEANO;
        }

	while(aux != NULL)
	{
		if ( strcmp(aux->info.tipo, UNARIO) == 0 ) //no caso do operador ser unrio
		{
                        ConsultaTipo(&aux->linkBack->info);

                        //se variavel j estiver marcada com INT
                        if( aux->info.id == SNAO )
		        {

                                ConsultaTipo(&aux->linkBack->info);

		        	//se variavel j estiver marcada com BOOL
		        	if( strcmp(aux->linkBack->info.lexema, "BOOL") == 0 )
		        	{
				        deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario pois o resultado j  booleano
		        	}
		        	else if( strstr(aux->linkBack->info.tipo, "BOOL") != NULL || aux->linkBack->info.id == SVERDADEIRO || aux->linkBack->info.id == SFALSO) //se for uma variavel inteiro  ou numero
		        	{
				        strcpy(aux->linkBack->info.lexema, "BOOL");//marca a variavel/numero como unria
                                        aux->linkBack->info.id == -1;
				        deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario
				
		        	}
		        	else
		        	{
                                         char error[256];
                                        sprintf(error, "Linha %d ERRO! Operador NAO deve ser precedido por um simbolo BOOLEANO",aux->info.line);
                                        strcpy(aux->info.error, error);
                                        return ERROR;
		        	}
	        	}
                        else if( strcmp(aux->linkBack->info.lexema, "INT") == 0 )
                        {
		         	deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario pois o resultado j  int
                        }
                        else if( strstr(aux->linkBack->info.tipo, "INT") != NULL || aux->linkBack->info.id == SNUMERO) //se for uma variavel inteiro  ou numero
                        {
		        	 strcpy(aux->linkBack->info.lexema, "INT");//marca a variavel/numero como unria
                                 aux->linkBack->info.id == -1;
		        	 deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario
                        }
                        else if ( strstr(aux->linkBack->info.tipo, "BOOL") != NULL || aux->linkBack->info.id == SVERDADEIRO || aux->linkBack->info.id == SFALSO)
                        {
                                 strcpy(aux->linkBack->info.lexema, "BOOL");//marca a variavel/numero como unria
                                 aux->linkBack->info.id == -1;
		        	 deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario
                        }
                        else
                        {
                                char error[256];
                                sprintf(error, "Linha %d ERRO! Operadores UNRIOS + ou - devem ser precedidos de um simbolo INTEIRO",aux->info.line);
                                strcpy(aux->info.error, error);
                                return ERROR;
                        }


		}
		else if(aux->info.id == SMAIS || aux->info.id == SMENOS || aux->info.id == SMULT || aux->info.id == SDIV)//operaes de comparao e aritmeticas
		{

                        ConsultaTipo(&aux->linkBack->info);

                        ConsultaTipo(&aux->linkBack->linkBack->info);
                        
			//se variavel j estiver marcada com INT
			if( strstr(aux->linkBack->info.lexema, "INT") != NULL )
			{	
				if( strcmp(aux->linkBack->linkBack->info.lexema, "INT") == 0 || strstr(aux->linkBack->linkBack->info.tipo, "INT") != NULL || aux->linkBack->linkBack->info.id == SNUMERO )
				{
					deletaItemExpressao(expressao, &aux->linkBack->linkBack->info); //deleta uma entrada INT deixando apenas uma (resultado)
					deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario pois o resultado j  int
				}
	
			}
			else if( strstr(aux->linkBack->info.tipo, "INT") != NULL || aux->linkBack->info.id == SNUMERO) //se for uma variavel inteiro  ou numero
			{
				if( strstr(aux->linkBack->linkBack->info.tipo, "INT") != NULL || aux->linkBack->linkBack->info.id == SNUMERO || strcmp(aux->linkBack->linkBack->info.lexema, "INT") == 0)
				{

					strcpy(aux->linkBack->info.lexema, "INT");//marca a variavel/numero como inteiro
                                        aux->linkBack->info.id == -1;
					deletaItemExpressao(expressao, &aux->linkBack->linkBack->info);
					deletaItemExpressao(expressao, &aux->info); //deleta o operador
					
				}
			}
			else
			{
				char error[256];
                                sprintf(error, "Linha %d ERRO! Operador '%s' deve ser utilizado entre simbolos INTEIROS",aux->info.line, aux->info.lexema);
                                strcpy(aux->info.error, error);
                                return ERROR;
			}
		}
		else if (aux->info.id == SMAIORIG || aux->info.id == SMAIOR || aux->info.id == SMENOR 
		  || aux->info.id == SIG || aux->info.id == SMENORIG || aux->info.id == SDIF)//operaes de comparao e aritmeticas
		{	

                        ConsultaTipo(&aux->linkBack->info);

                        ConsultaTipo(&aux->linkBack->linkBack->info);

			if( strcmp(aux->linkBack->info.lexema, "INT") == 0 )
			{	
				if( strcmp(aux->linkBack->linkBack->info.lexema, "INT") == 0 || strstr(aux->linkBack->linkBack->info.tipo, "INT") != NULL || aux->linkBack->linkBack->info.id == SNUMERO )
				{
					strcpy(aux->linkBack->info.lexema, "BOOL");
                                        aux->linkBack->info.id == -1;
					deletaItemExpressao(expressao, &aux->linkBack->linkBack->info); //deleta uma entrada INT deixando apenas uma (resultado)
					deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario pois o resultado j  int	
				}
	
			}
			else if( strstr(aux->linkBack->info.tipo, "INT") != NULL || aux->linkBack->info.id == SNUMERO ) //se for uma variavel inteiro  ou numero
			{
				if( strstr(aux->linkBack->linkBack->info.tipo, "INT") != NULL || aux->linkBack->linkBack->info.id == SNUMERO || strcmp(aux->linkBack->linkBack->info.lexema, "INT") == 0)
				{
					strcpy(aux->linkBack->info.lexema, "BOOL");//marca a variavel/numero como inteiro
                                        aux->linkBack->info.id == -1;
					deletaItemExpressao(expressao, &aux->linkBack->linkBack->info);
					deletaItemExpressao(expressao, &aux->info); //deleta o operador
					
				}
			}
			else
			{
				char error[256];
                                sprintf(error, "Linha %d ERRO! Operador '%s' deve utilizado entre dois simbolos INTEIROS",aux->info.line, aux->info.lexema);
                                strcpy(aux->info.error, error);
                                return ERROR;
			}
		}
		else if (aux->info.id == SE || aux->info.id == SOU )//operaes de comparao e aritmeticas
		{	


                        ConsultaTipo(&aux->linkBack->info);
                        ConsultaTipo(&aux->linkBack->linkBack->info);

			if( strcmp(aux->linkBack->info.lexema, "BOOL") == 0 )
			{	
				if( strcmp(aux->linkBack->linkBack->info.lexema, "BOOL") == 0 || strstr(aux->linkBack->linkBack->info.tipo, "BOOL") != NULL)
				{		
					deletaItemExpressao(expressao, &aux->linkBack->linkBack->info); //deleta uma entrada INT deixando apenas uma (resultado)
					deletaItemExpressao(expressao, &aux->info); //deleta o simbolo unario pois o resultado j  int
				}
	
			}
			else if(strstr(aux->linkBack->info.tipo, "BOOL") != NULL ) //se for uma variavel inteiro  ou numero
			{
				if( strstr(aux->linkBack->linkBack->info.tipo, "BOOL") != NULL || strcmp(aux->linkBack->linkBack->info.lexema, "BOOL") == 0)
				{
					strcpy(aux->linkBack->linkBack->info.lexema, "BOOL");//marca a variavel/numero como inteiro
                                        aux->linkBack->linkBack->info.id == -1;
					deletaItemExpressao(expressao, &aux->linkBack->info);
					deletaItemExpressao(expressao, &aux->info); //deleta o operador

				}
			}
			else
			{
				char error[256];
                                sprintf(error, "Linha %d ERRO! Operador '%s' deve utilizado entre dois simbolos BOOLEANOS",aux->info.line, aux->info.lexema);
                                strcpy(aux->info.error, error);
                                return ERROR;
			}
		}
		
		aux = aux->link;

	}

	if( strcmp(expressao.top->info.lexema, "INT") == 0 )
	{
		return SINTEIRO;
	}
	else
		return SBOOLEANO;

}


/*
	Funcao gera pos fixo
*/
void GeraCodigoPosFixo()
{

	no *inicio_expressao;
	Pilha expressao; 
	int endereco_memoria, rotulo;

	init(expressao);

	DuplicarInvPilha(saida, expressao);

	inicio_expressao = expressao.top;

	inicio_expressao->linkBack = NULL;


	while(inicio_expressao != NULL)
	{
		if(inicio_expressao->info.id == SIDENTIFICADOR )
		{
			if( ConsultaTipo(&inicio_expressao->info) == VARIAVEL)
			{
				endereco_memoria = ConsultaEndereco(inicio_expressao->info.lexema);

				GeraCodigo(-1,"LDV",endereco_memoria,-1);
			}


			if( ConsultaTipo(&inicio_expressao->info) == FUNCAO)
			{
				rotulo = ConsultaRotulo(inicio_expressao->info.lexema);

				GeraCodigo(-1,"CALL",rotulo,-1);
                                GeraCodigo(-1,"LDV",0,-1);
			}
                        
		}
		else if ( inicio_expressao->info.id == SNUMERO )
		{
			GeraCodigo(-1,"LDC",atoi(inicio_expressao->info.lexema),-1);

		}
		else if( strcmp(inicio_expressao->info.tipo, UNARIO) == 0 )
		{
			//inverte apenas se for negativo.
			if( strcmp(inicio_expressao->info.lexema, "-") == 0)
			{
				GeraCodigo(-1,"INV",-1,-1);
			}
                        else if(strcmp(inicio_expressao->info.lexema, "nao") == 0)
                        {
                               GeraCodigo(-1,"NEG",-1,-1);
                        }
		}
		else if( inicio_expressao->info.id == SMAIS )
		{
			GeraCodigo(-1,"ADD",-1,-1);
		}
		else if( inicio_expressao->info.id == SMENOS )
		{
			GeraCodigo(-1,"SUB",-1,-1);
		}
		else if( inicio_expressao->info.id == SMULT )
		{
			GeraCodigo(-1,"MULT",-1,-1);
		}
		else if( inicio_expressao->info.id == SDIV )
		{
			GeraCodigo(-1,"DIVI",-1,-1);
		}
		else if( inicio_expressao->info.id == SMAIOR )
		{
			GeraCodigo(-1,"CMA",-1,-1);
		}
		else if( inicio_expressao->info.id == SMENOR )
		{
			GeraCodigo(-1,"CME",-1,-1);
		}
		else if( inicio_expressao->info.id == SMAIORIG )
		{
			GeraCodigo(-1,"CMAQ",-1,-1);
		}
		else if( inicio_expressao->info.id == SMENORIG )
		{
			GeraCodigo(-1,"CMEQ",-1,-1);
		}
		else if( inicio_expressao->info.id == SE )
		{
			GeraCodigo(-1,"AND",-1,-1);
		}
		else if( inicio_expressao->info.id == SOU )
		{
			GeraCodigo(-1,"OR",-1,-1);
		}
		else if( inicio_expressao->info.id == SIG )
		{
			GeraCodigo(-1,"CEQ",-1,-1);
		}
		else if( inicio_expressao->info.id == SDIF )
		{
			GeraCodigo(-1,"CDIF",-1,-1);
		}
                else if( inicio_expressao->info.id == SVERDADEIRO )
                {
                        GeraCodigo(-1,"LDC",1,-1);
                }
                else if( inicio_expressao->info.id == SFALSO )
                {
                        GeraCodigo(-1,"LDC",0,-1);
                }

		inicio_expressao = inicio_expressao->link;
	}

	//limpa lista saida
	while(saida.top != NULL)
	{
		pop(saida);
	}
}

void CloseSintaticoFile()
{
        fclose(arquivo);
}
